local _G = getfenv(0);

local modName = "QButtons";
local shownItems;
local slots = {
	"HeadSlot", "NeckSlot", "ShoulderSlot", "BackSlot", "ChestSlot", "ShirtSlot", "TabardSlot", "WristSlot",
	"HandsSlot", "WaistSlot", "LegsSlot", "FeetSlot", "Finger0Slot", "Finger1Slot", "Trinket0Slot", "Trinket1Slot",
	"MainHandSlot", "SecondaryHandSlot", "RangedSlot",
};

-- Config
QButtons_Config = {};
local cfg;
local defConfig = {
	enabled = true,
	size = 32,
	showTips = true,
	vertical = false,
};

--------------------------------------------------------------------------------------------------------
--                                                Main                                                --
--------------------------------------------------------------------------------------------------------

local f = CreateFrame("Frame","QButtons",UIParent);
f:SetBackdrop({ bgFile = "Interface\\ChatFrame\\ChatFrameBackground", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = 1, tileSize = 16, edgeSize = 16, insets = { left = 4, right = 4, top = 4, bottom = 4 } });
f:SetBackdropColor(0.1,0.22,0.35,1);
f:SetBackdropBorderColor(0.1,0.1,0.1,1);
f:EnableMouse(1);
f:SetMovable(1);
f:SetToplevel(1);
f:SetClampedToScreen(1);
f:SetFrameStrata("DIALOG");

local function OnMouseUp(self,button)
	f:StopMovingOrSizing();
	cfg.left, cfg.bottom = f:GetLeft(), f:GetBottom();
end

f:SetScript("OnMouseDown",function(self,button) f:StartMoving(); end);
f:SetScript("OnMouseUp",OnMouseUp);

f.tip = CreateFrame("GameTooltip","QButtonsTip",nil,"GameTooltipTemplate");
f.tip:SetOwner(UIParent,"ANCHOR_NONE");

local function SetEnabledStatus()
	if (cfg.enabled) then
		f:RegisterEvent("BAG_UPDATE");
		f:RegisterEvent("UNIT_INVENTORY_CHANGED");
		f:RegisterEvent("PLAYER_REGEN_ENABLE");

		f:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");

		if (not InCombatLockdown()) then
			f.UpdateButtons();
		end
	else
		f:UnregisterAllEvents();
		f:Hide();
	end
end

--------------------------------------------------------------------------------------------------------
--                                               Events                                               --
--------------------------------------------------------------------------------------------------------
local function OnEvent(self,event,...)
	-- Variables Loaded
	if (event == "VARIABLES_LOADED") then
		cfg = QButtons_Config;
		if (cfg.left and cfg.bottom) then
			f:SetPoint("BOTTOMLEFT",cfg.left,cfg.bottom);
		else
			f:SetPoint("CENTER");
		end
		for option, value in pairs(defConfig) do
			if (cfg[option] == nil) then
				cfg[option] = value;
			end
		end
		SetEnabledStatus();
	-- Update Cooldowns
	elseif (event == "ACTIONBAR_UPDATE_COOLDOWN") and (shownItems > 0) then
		f.UpdateCooldowns();
	-- Other Events
	elseif (not InCombatLockdown()) then
		f.UpdateButtons();
	end
end

f:SetScript("OnEvent",OnEvent);
f:RegisterEvent("VARIABLES_LOADED");

--------------------------------------------------------------------------------------------------------
--                                            Command Line                                            --
--------------------------------------------------------------------------------------------------------
_G["SLASH_"..modName.."1"] = "/qb";
SlashCmdList[modName] = function(cmd)
	-- Extract Parameters
	local param1, param2 = cmd:match("^([^%s]+)%s*(.*)$");
	param1 = (param1 and param1:lower() or cmd:lower());
	-- Enabled
	if (param1 == "toggle") then
		cfg.enabled = not cfg.enabled;
		AzMsg("|2"..modName.."|r has now been |1"..(cfg.enabled and "enabled" or "disabled").."|r.");
		SetEnabledStatus();
	-- Size
	elseif (param1 == "size") then
		if (param2 ~= "") and (type(tonumber(param2)) == "number") then
			cfg.size = tonumber(param2);
			for _, button in ipairs(f.items) do
				button:SetWidth(cfg.size);
				button:SetHeight(cfg.size);
			end
			AzMsg("Button size set to |1"..cfg.size.."|r.");
			f.UpdateButtons();
		end
	-- Tips
	elseif (param1 == "tips") then
		cfg.showTips = not cfg.showTips;
		AzMsg("Item tips |1"..(cfg.showTips and "enabled" or "disabled").."|r.");
	-- Vertical
	elseif (param1 == "vertical") then
		cfg.vertical = not cfg.vertical;
		AzMsg("Vertical alignment |1"..(cfg.vertical and "enabled" or "disabled").."|r.");
		f.UpdateButtons();
	-- Invalid or No Command
	else
		UpdateAddOnMemoryUsage();
		AzMsg(format("----- |2%s|r |1%s|r ----- |1%.2f |2kb|r -----",modName,GetAddOnMetadata(modName,"Version"),GetAddOnMemoryUsage(modName)));
		AzMsg("The following |2parameters|r are valid for this addon:");
		AzMsg(" |2toggle|r = Toggles the mod being on or off");
		AzMsg(" |2size 'value'|r = Sets the size of the icons");
		AzMsg(" |2tips|r = Toggles the showing of item tips when you mouse over buttons");
		AzMsg(" |2vertical|r = Sets whether or not the buttons anchor vertical or horizontal");
	end
end
--------------------------------------------------------------------------------------------------------
--                                                Items                                               --
--------------------------------------------------------------------------------------------------------

f.items = {};

-- Button Scripts
local function Button_OnEnter(self)
	if (cfg.showTips) then
		GameTooltip:SetOwner(self,"ANCHOR_RIGHT");
		local bag, slot = self:GetAttribute("bag"), self:GetAttribute("slot");
		if (bag) then
			GameTooltip:SetBagItem(bag,slot);
		else
			GameTooltip:SetInventoryItem("player",slot);
		end
	end
end

local function HideGTT()
	GameTooltip:Hide();
end

-- Make Loot Button
local function MakeItemButton()
	local b = CreateFrame("Button",nil,f,"SecureActionButtonTemplate");
	b:SetWidth(cfg.size);
	b:SetHeight(cfg.size);
	b:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square");
	b:RegisterForClicks("LeftButtonUp","RightButtonUp");
	b:SetScript("OnEnter",Button_OnEnter);
	b:SetScript("OnLeave",HideGTT);
	b:SetAttribute("type*","item");

	b.icon = b:CreateTexture(nil,"ARTWORK");
	b.icon:SetAllPoints();

	b.count = b:CreateFontString(nil,"ARTWORK","NumberFontNormal");
	b.count:SetPoint("BOTTOMRIGHT",b.icon,-3,3);

	b.cooldown = CreateFrame("Cooldown",nil,b,"CooldownFrameTemplate");
	b.cooldown:SetAllPoints();

	if (#f.items == 0) then
		b:SetPoint("TOPLEFT",f,8,-8);
	end

	tinsert(f.items,b);
	return b;
end

-- Add Button
local function AddButton(index,bag,slot,link,count)
	button = f.items[index] or MakeItemButton();

	button.icon:SetTexture(select(10,GetItemInfo(link)));
	button.count:SetText(count and count > 1 and count or "");

	button:SetAttribute("bag",bag);
	button:SetAttribute("slot",slot);

	if (index > 1) then
		button:ClearAllPoints();
		if (cfg.vertical) then
			button:SetPoint("TOP",f.items[index - 1],"BOTTOM",0,-1);
		else
			button:SetPoint("LEFT",f.items[index - 1],"RIGHT",1,0);
		end
	end

	button:Show();
end

-- Check Item
local function CheckItem(link)
	f.tip:ClearLines();
	f.tip:SetHyperlink(link);
	if (f.tip:NumLines() >= 3) and (_G["QButtonsTipTextLeft2"]:GetText() == ITEM_BIND_QUEST) then
		for i = 3, f.tip:NumLines() do
			if (_G["QButtonsTipTextLeft"..i]:GetText() or ""):find("^"..ITEM_SPELL_TRIGGER_ONUSE) then
				return 1;
			end
		end
	end
	return;
end

-- Update Buttons
function f.UpdateButtons()
	local link, slotId;
	local index = 1;
	-- Inventory
	for bag = 0, NUM_BAG_FRAMES do
		for slot = 1, GetContainerNumSlots(bag) do
			link = GetContainerItemLink(bag,slot);
			if (link) and (CheckItem(link)) then
				AddButton(index,bag,slot,link,select(2,GetContainerItemInfo(bag,slot)));
				index = (index + 1);
			end
		end
	end
	-- Equipped Items
	for _, slotName in ipairs(slots) do
		slotId = GetInventorySlotInfo(slotName);
		link = GetInventoryItemLink("player",slotId);
		if (link) and (CheckItem(link)) then
			AddButton(index,nil,slotId,link);
			index = (index + 1);
		end
	end
	-- Hide/Show depending on we have items showing
	shownItems = (index - 1);
	if (shownItems == 0) then
		f:Hide();
	else
		f:Show();
		-- Hide the Rest
		for i = index, #f.items do
			f.items[i]:Hide();
		end
		-- Set Frame Dimention
		local x = ((cfg.size + 1) * shownItems + 15);
		local y = (cfg.size + 16);
		f:SetWidth(cfg.vertical and y or x);
		f:SetHeight(cfg.vertical and x or y);
	end
end

-- Update Cooldowns
function f.UpdateCooldowns()
	local bag, slot;
	for i = 1, shownItems do
		bag, slot = f.items[i]:GetAttribute("bag"), f.items[i]:GetAttribute("slot");
		if (bag) then
			CooldownFrame_SetTimer(f.items[i].cooldown,GetContainerItemCooldown(bag,slot));
		else
			CooldownFrame_SetTimer(f.items[i].cooldown,GetInventoryItemCooldown("player",slot));
		end
	end
end

--------------------------------------------------------------------------------------------------------
--                                Global Chat Message Function (Rev 3)                                --
--------------------------------------------------------------------------------------------------------
if (not AZMSG_REV or AZMSG_REV < 3) then
	AZMSG_REV = 3;
	function AzMsg(text)
		DEFAULT_CHAT_FRAME:AddMessage(tostring(text):gsub("|1","|cffffff80"):gsub("|2","|cffffffff"),128/255,192/255,255/255);
	end
end